@@ -226,7 +226,7 @@ use strum_macros::IntoStaticStr;
 use tempfile;
 
 use crate::{
-    config::{Config, Delimiter},
+    config::{Config, Delimiter, DEFAULT_WTR_BUFFER_CAPACITY},
     util, CliError, CliResult,
 };
 
@@ -656,7 +656,7 @@ fn sequential_mode(
 
     #[cfg(any(feature = "feature_capable", feature = "lite"))]
     let show_progress =
-        (args.flag_progressbar || std::env::var("QSV_PROGRESSBAR").is_ok()) && !rconfig.is_stdin();
+        (args.flag_progressbar || util::get_envvar_flag("QSV_PROGRESSBAR")) && !rconfig.is_stdin();
     #[cfg(any(feature = "feature_capable", feature = "lite"))]
     let progress = ProgressBar::with_draw_target(None, ProgressDrawTarget::stderr_with_hz(5));
     #[cfg(any(feature = "feature_capable", feature = "lite"))]
@@ -986,7 +986,7 @@ fn random_acess_mode(
 
     #[cfg(any(feature = "feature_capable", feature = "lite"))]
     let show_progress =
-        (args.flag_progressbar || std::env::var("QSV_PROGRESSBAR").is_ok()) && !rconfig.is_stdin();
+        (args.flag_progressbar || util::get_envvar_flag("QSV_PROGRESSBAR")) && !rconfig.is_stdin();
     #[cfg(any(feature = "feature_capable", feature = "lite"))]
     let progress = ProgressBar::with_draw_target(None, ProgressDrawTarget::stderr_with_hz(5));
     #[cfg(any(feature = "feature_capable", feature = "lite"))]
@@ -1350,7 +1350,8 @@ fn create_index(arg_input: &Option<String>) -> Result<bool, CliError> {
 
     let rconfig = Config::new(&Some(input.to_string()));
     let mut rdr = rconfig.reader_file()?;
-    let mut wtr = io::BufWriter::new(fs::File::create(pidx)?);
+    let mut wtr =
+        io::BufWriter::with_capacity(DEFAULT_WTR_BUFFER_CAPACITY, fs::File::create(pidx)?);
     if RandomAccessSimple::create(&mut rdr, &mut wtr).is_err() {
         return Ok(false);
     };
@@ -1914,11 +1915,20 @@ fn setup_helpers(
                 mv, nl, pwd, sed, sort, tail, touch, tr, uname, uniq, wc, whoami")
         };
 
-        let mut cmd = std::process::Command::new(shellcmd_string.clone());
         let args_string = args.as_str().to_string();
         let args_vec: Vec<&str> = args_string.split_whitespace().collect();
         log::info!("Invoking qsv_shellcmd: {shellcmd_string} {args_string}");
-        let result = cmd.args(args_vec).output();
+
+        let result = if cfg!(target_os = "windows") {
+            std::process::Command::new("cmd")
+                .args(["/C", &shellcmd_string])
+                .args(args_vec)
+                .output()
+        } else {
+            std::process::Command::new(shellcmd_string.clone())
+                .args(args_vec)
+                .output()
+        };
 
         match result {
             Ok(output) => {
@@ -2075,7 +2085,9 @@ fn setup_helpers(
                 let client_timeout = std::time::Duration::from_secs(TIMEOUT_SECS.load(Ordering::Relaxed) as u64);
 
                 let client = match Client::builder()
-                    .user_agent(util::DEFAULT_USER_AGENT)
+                    // safety: we're using a validated QSV_USER_AGENT or if it's not set,
+                    // the default user agent                    
+                    .user_agent(util::set_user_agent(None).unwrap())
                     .brotli(true)
                     .gzip(true)
                     .deflate(true)
