@@ -7,7 +7,9 @@ field names & data types) using a Viterbi algorithm.
 
 On Linux, `sniff` also acts as a general mime type detector using the libmagic library,
 returning the detected mime type, file size and last modified date if the file is not
-a CSV. If --no-infer is enabled, it doesn't even bother to infer the CSV's schema.
+a CSV. If --no-infer is enabled, it doesn't even bother to infer the CSV's schema. This
+makes it especially useful for accelerated CKAN harvesting and for checking stale/broken
+resource URLs.
 
 On macOS and Windows however, `sniff` is only a CSV dialect detector and does not
 detect other file types. It can only sniff files with the "csv", "tsv", "tab" and
@@ -42,7 +44,8 @@ sniff arguments:
                              CSV schema and just work as a general mime type detector -
                              returning the detected mime type, file size and last modified date.
                              When sniffing a local file, it will scan the file to detect the mime type.
-                             When sniffing a URL, it will only download the first chunk of the file.
+                             When sniffing a URL, it will only download the first chunk of the file if 
+                             the --quick option is enabled, otherwise it will download the entire file.
                              
 sniff options:
     --sample <size>          First n rows to sample to sniff out the metadata.
@@ -62,21 +65,30 @@ sniff options:
                              Valid only when input is a URL.
     --timeout <secs>         Timeout when sniffing URLs in seconds.
                              [default: 30]
-    --user-agent <agent>     Specify a custom user agent to use when sniffing a CSV on a URL.
-                             Try to follow the syntax here -
+    --user-agent <agent>     Specify custom user agent to use when sniffing a CSV on a URL.
+                             It supports the following variables - $QSV_VERSION, $QSV_TARGET,
+                             $QSV_BIN_NAME and $QSV_KIND. Try to follow the syntax here -
                              https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/User-Agent
     --stats-types            Use the same data type names as `stats`.
                              (Unsigned, Signed => Integer, Text => String, everything else the same)
     --no-infer               Do not infer the schema. Only return the file's mime type, size and
                              last modified date. Use this to use sniff as a general mime type detector.
                              Valid only on Linux.
+    --quick                  When sniffing a non-CSV remote file, only download the first chunk of the file
+                             before attempting to detect the mime type. This is faster but less accurate as
+                             some mime types cannot be detected with just the first chunk.
+                             Valid only on Linux.
+    --harvest-mode           This is a convenience flag when using sniff in CKAN harvesters. 
+                             It is equivalent to --quick --timeout 10 --stats-types --json
+                             and --user-agent "CKAN-harvest/$QSV_VERSION ($QSV_TARGET; $QSV_BIN_NAME)"
+                             Valid only on Linux.
 
 Common options:
     -h, --help               Display this message
     -d, --delimiter <arg>    The field delimiter for reading CSV data.
                              Specify this when the delimiter is known beforehand,
                              as the delimiter guessing algorithm can sometimes be
-                             wrong if not enough delimiters are present in the sample.
+                               wrong if not enough delimiters are present in the sample.
                              Must be a single ascii character.
     -p, --progressbar        Show progress bars. Only valid for URL input.
 "#;
@@ -84,7 +96,7 @@ Common options:
 use std::{
     cmp::min,
     fmt, fs,
-    io::Write,
+    io::{copy, Seek, SeekFrom, Write},
     path::PathBuf,
     time::{Duration, SystemTime},
 };
@@ -97,7 +109,7 @@ use indicatif::{HumanBytes, HumanCount, ProgressBar, ProgressDrawTarget, Progres
 use qsv_sniffer::{DatePreference, SampleSize, Sniffer};
 use reqwest::Client;
 use serde::{Deserialize, Serialize};
-use serde_json::json;
+use serde_json::{json, Value};
 use tabwriter::TabWriter;
 use tempfile::NamedTempFile;
 use thousands::Separable;
@@ -122,6 +134,8 @@ struct Args {
     flag_user_agent:     Option<String>,
     flag_stats_types:    bool,
     flag_no_infer:       bool,
+    flag_quick:          bool,
+    flag_harvest_mode:   bool,
 }
 
 #[derive(Serialize, Deserialize, Default, Debug)]
@@ -253,6 +267,8 @@ impl fmt::Display for SniffStruct {
 struct SniffFileStruct {
     display_path:       String,
     file_to_sniff:      String,
+    #[cfg(all(target_os = "linux", feature = "magic"))]
+    detected_mime:      String,
     tempfile_flag:      bool,
     retrieved_size:     usize,
     file_size:          usize,
@@ -403,13 +419,9 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                 #[allow(unused_assignments)]
                 let mut chunk = Bytes::new(); // amortize the allocation
 
-                // on linux, we can short-circuit downloading by
-                // checking the file type from the first chunk
                 // the unused_muts are here to suppress warnings
                 // when the magic feature is not enabled
                 #[allow(unused_mut)]
-                let mut shortcircuit_flag = false;
-                #[allow(unused_mut)]
                 let mut firstchunk = Bytes::new();
                 #[allow(unused_mut)]
                 let mut magic_flag = false;
@@ -430,11 +442,12 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
 
                     // on linux, we can short-circuit downloading
                     // by checking the file type from the first chunk
+                    // when --quick is enabled
                     #[cfg(all(target_os = "linux", feature = "magic"))]
-                    if downloaded == 0 && !snappy_flag {
+                    if downloaded == 0 && !snappy_flag && args.flag_quick {
                         let mime = util::sniff_filetype_from_buffer(&chunk)?;
+                        log::debug!("scanned first {chunk_len} bytes - detected mime: {mime}");
                         if !mime.starts_with("text/") && mime != "application/csv" {
-                            shortcircuit_flag = true;
                             downloaded = chunk_len;
                             firstchunk = chunk.clone();
                             break;
@@ -463,6 +476,7 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                     }
                 }
                 drop(client);
+                file.flush()?;
 
                 #[cfg(any(feature = "feature_capable", feature = "lite"))]
                 if show_progress {
@@ -479,6 +493,22 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                     }
                 }
 
+                let wtr_file_path;
+                #[allow(unused_mut)]
+                let mut csv_candidate = true;
+                #[allow(unused_mut)]
+                #[allow(unused_assignments)]
+                #[allow(unused_variables)]
+                let mut detected_mime = String::new();
+
+                #[cfg(all(target_os = "linux", feature = "magic"))]
+                if !args.flag_quick {
+                    detected_mime =
+                        util::get_filetype(&file.path().to_path_buf().display().to_string())?;
+                    csv_candidate =
+                        detected_mime.starts_with("text/") || detected_mime == "application/csv";
+                }
+
                 // create a temporary file to write the download file to
                 let wtr_file = NamedTempFile::new()?;
 
@@ -487,7 +517,6 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                 let (mut tmp_file, path) =
                     wtr_file.keep().map_err(|_| "Cannot keep temporary file")?;
 
-                let wtr_file_path;
                 let mut downloaded_records = 0_usize;
 
                 if snappy_flag {
@@ -495,14 +524,14 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                     // before we can sniff it
                     wtr_file_path =
                         util::decompress_snappy_file(&file.path().to_path_buf(), tmpdir)?;
-                } else if shortcircuit_flag {
-                    // on linux, we can short-circuit downloading by checking the file type
-                    // from the first chunk. If the file is not a CSV file, we just write
-                    // the first chunk to a file and return
+                } else if args.flag_quick && magic_flag {
+                    // on linux, when --quiick is enabled, we short-circuit downloading by checking
+                    // the file type from the first chunk. If the file is not a CSV,
+                    // we just write the first chunk to a file and return
                     wtr_file_path = path.display().to_string();
                     tmp_file.write_all(&firstchunk)?;
                     tmp_file.flush()?;
-                } else {
+                } else if csv_candidate {
                     // we downloaded a non-snappy file and it might be a CSV file.
                     // Rewrite it so we only have the exact sample size and truncate potentially
                     // incomplete lines. We do this coz we streamed the download and the downloaded
@@ -542,11 +571,20 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                         wtr.write_byte_record(&record)?;
                     }
                     wtr.flush()?;
+                } else {
+                    // we downloaded a non-snappy file and its not a CSV file
+                    // just copy it over
+                    wtr_file_path = path.display().to_string();
+                    file.seek(SeekFrom::Start(0))?;
+                    copy(&mut file, &mut tmp_file)?;
+                    log::debug!("Not a CSV file, just copy it over");
                 }
 
                 Ok(SniffFileStruct {
                     display_path: url,
                     file_to_sniff: wtr_file_path,
+                    #[cfg(all(target_os = "linux", feature = "magic"))]
+                    detected_mime,
                     tempfile_flag: true,
                     retrieved_size: downloaded,
                     file_size: if magic_flag && total_size == usize::MAX {
@@ -637,6 +675,8 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                 Ok(SniffFileStruct {
                     display_path: canonical_path,
                     file_to_sniff: path,
+                    #[cfg(all(target_os = "linux", feature = "magic"))]
+                    detected_mime: "".to_string(),
                     tempfile_flag: false,
                     retrieved_size: file_size,
                     file_size,
@@ -675,6 +715,8 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
         Ok(SniffFileStruct {
             display_path: "stdin".to_string(),
             file_to_sniff: path_string,
+            #[cfg(all(target_os = "linux", feature = "magic"))]
+            detected_mime: "".to_string(),
             tempfile_flag: true,
             retrieved_size: file_size,
             file_size,
@@ -694,20 +736,38 @@ fn cleanup_tempfile(
     Ok(())
 }
 
-#[allow(clippy::unused_async)] // false positive lint
-pub async fn run(argv: &[&str]) -> CliResult<()> {
+pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
 
+    let rt = tokio::runtime::Runtime::new().unwrap();
+    rt.block_on(sniff_main(args))?;
+
+    Ok(())
+}
+
+#[allow(clippy::unused_async)] // false positive lint
+async fn sniff_main(mut args: Args) -> CliResult<()> {
+    if args.flag_harvest_mode {
+        args.flag_quick = true;
+        args.flag_timeout = 10;
+        args.flag_stats_types = true;
+        args.flag_json = true;
+        args.flag_user_agent =
+            Some("CKAN-harvest/$QSV_VERSION ($QSV_TARGET; $QSV_BIN_NAME)".to_string());
+    }
+
     #[cfg(not(target_os = "linux"))]
-    if args.flag_no_infer {
+    if args.flag_no_infer || args.flag_quick {
         return fail_clierror!(
-            "--no-infer is only supported on Linux with the 'magic' feature enabled"
+            "--no-infer and --quick are only supported on Linux with the 'magic' feature enabled"
         );
     }
 
     #[cfg(all(target_os = "linux", not(feature = "magic")))]
-    if args.flag_no_infer {
-        return fail_clierror!("--no-infer is only supported when the 'magic' feature is enabled");
+    if args.flag_no_infer || args.flag_quick {
+        return fail_clierror!(
+            "--no-infer and --quick are only supported when the 'magic' feature is enabled"
+        );
     }
 
     let mut sample_size = args.flag_sample;
@@ -730,72 +790,91 @@ pub async fn run(argv: &[&str]) -> CliResult<()> {
     let future = get_file_to_sniff(&args, &tmpdir);
     let sfile_info = block_on(future)?;
     let tempfile_to_delete = sfile_info.file_to_sniff.clone();
+    #[allow(unused_assignments)]
+    #[allow(unused_mut)]
+    #[allow(unused_variables)]
+    let mut file_type = String::new();
 
     // on linux, check what kind of file we have
     // if its NOT a CSV or a text file, we fail, showing the detected mime type
     #[cfg(all(target_os = "linux", feature = "magic"))]
-    let file_type = util::get_filetype(&sfile_info.file_to_sniff)?;
-    #[cfg(all(target_os = "linux", feature = "magic"))]
-    // we also accept text/* files, as sniff may still be able to suss out
-    // if the file is a CSV, even if its not using a CSV extension
-    // we can also be here if the user has specified --no-infer
-    if (file_type != "application/csv" && !file_type.starts_with("text/")) || args.flag_no_infer {
-        cleanup_tempfile(sfile_info.tempfile_flag, tempfile_to_delete)?;
-
-        let size = if sfile_info.file_size >= usize::MAX - 1 {
-            "Unknown".to_string()
+    {
+        // if we have a detected mime type earlier, we can skip the sniffing
+        // unless it was a snappy file and --no-infer is disabled,
+        // as we need to sniff the uncompressed file
+        if sfile_info.detected_mime.is_empty()
+            || sfile_info.detected_mime == "application/x-snappy-framed" && !args.flag_no_infer
+        {
+            file_type = util::get_filetype(&sfile_info.file_to_sniff)?;
         } else {
-            sfile_info.file_size.to_string()
-        };
-        let last_modified = sfile_info.last_modified.clone();
+            file_type = sfile_info.detected_mime.clone();
+        }
 
-        if args.flag_json || args.flag_pretty_json {
-            if args.flag_no_infer {
-                let json_result = json!({
-                    "title": "sniff mime type",
-                    "meta": {
-                        "detected_mime_type": file_type,
-                        "size": size,
-                        "last_modified": last_modified,
-                    }
-                });
-                if args.flag_pretty_json {
-                    woutinfo!("{}", serde_json::to_string_pretty(&json_result).unwrap());
-                    return Ok(());
-                }
-                woutinfo!("{json_result}");
-                return Ok(());
+        // we also accept text/* files, as sniff may still be able to suss out
+        // if the file is a CSV, even if its not using a CSV extension
+        // we can also be here if the user has specified --no-infer
+        if (file_type != "application/csv" && !file_type.starts_with("text/")) || args.flag_no_infer
+        {
+            cleanup_tempfile(sfile_info.tempfile_flag, tempfile_to_delete)?;
+
+            let size = if sfile_info.file_size >= usize::MAX - 1 {
+                "Unknown".to_string()
             } else {
-                let json_result = json!({
-                    "errors": [{
-                        "title": "sniff error",
-                        "detail": format!("File is not a CSV file. Detected mime type: {file_type}"),
+                sfile_info.file_size.to_string()
+            };
+            let mut last_modified = sfile_info.last_modified;
+            if last_modified.is_empty() {
+                last_modified = "Unknown".to_string();
+            }
+
+            if args.flag_json || args.flag_pretty_json {
+                if args.flag_no_infer {
+                    let json_result = json!({
+                        "title": "sniff mime type",
                         "meta": {
                             "detected_mime_type": file_type,
                             "size": size,
                             "last_modified": last_modified,
                         }
-                    }]
-                });
-                if args.flag_pretty_json {
-                    return fail_clierror!(
-                        "{}",
-                        serde_json::to_string_pretty(&json_result).unwrap()
-                    );
+                    });
+                    if args.flag_pretty_json {
+                        woutinfo!("{}", serde_json::to_string_pretty(&json_result).unwrap());
+                        return Ok(());
+                    }
+                    woutinfo!("{json_result}");
+                    return Ok(());
+                } else {
+                    let json_result = json!({
+                        "errors": [{
+                            "title": "sniff error",
+                            "detail": format!("File is not a CSV file. Detected mime type: {file_type}"),
+                            "meta": {
+                                "detected_mime_type": file_type,
+                                "size": size,
+                                "last_modified": last_modified,
+                            }
+                        }]
+                    });
+                    if args.flag_pretty_json {
+                        return fail_clierror!(
+                            "{}",
+                            serde_json::to_string_pretty(&json_result).unwrap()
+                        );
+                    }
+                    return fail_clierror!("{json_result}");
                 }
-                return fail_clierror!("{json_result}");
             }
-        }
-        if args.flag_no_infer {
-            woutinfo!(
-                "Detected mime type: {file_type}, size: {size}, last modified: {last_modified}"
-            );
-            return Ok(());
-        } else {
-            return fail_clierror!(
-                "File is not a CSV file. Detected mime type: {file_type}, size: {size}, last \
-                 modified: {last_modified}"
-            );
+            if args.flag_no_infer {
+                woutinfo!(
+                    "Detected mime type: {file_type}, size: {size}, last modified: {last_modified}"
+                );
+                return Ok(());
+            } else {
+                return fail_clierror!(
+                    "File is not a CSV file. Detected mime type: {file_type}, size: {size}, last \
+                     modified: {last_modified}"
+                );
+            }
         }
     }
 
@@ -915,7 +994,7 @@ pub async fn run(argv: &[&str]) -> CliResult<()> {
     };
 
     let mut processed_results = SniffStruct::default();
-    let mut sniffing_error: Option<String> = None;
+    let mut sniff_error: Option<String> = None;
 
     match sniff_results {
         Ok(metadata) => {
@@ -935,7 +1014,7 @@ pub async fn run(argv: &[&str]) -> CliResult<()> {
             processed_results = SniffStruct {
                 path: sfile_info.display_path,
                 sniff_timestamp: sniffed_ts,
-                last_modified: sfile_info.last_modified,
+                last_modified: sfile_info.last_modified.clone(),
                 delimiter_char: metadata.dialect.delimiter as char,
                 header_row: metadata.dialect.header.has_header_row,
                 preamble_rows: metadata.dialect.header.num_preamble_rows,
@@ -946,9 +1025,9 @@ pub async fn run(argv: &[&str]) -> CliResult<()> {
                 flexible: metadata.dialect.flexible,
                 is_utf8: metadata.dialect.is_utf8,
                 #[cfg(all(target_os = "linux", feature = "magic"))]
-                detected_mime: file_type,
+                detected_mime: file_type.clone(),
                 retrieved_size: sfile_info.retrieved_size,
-                file_size: sfile_info.file_size, // sfile_info.file_size,
+                file_size: sfile_info.file_size,
                 sampled_records: if sampled_records > num_records {
                     num_records
                 } else {
@@ -964,21 +1043,15 @@ pub async fn run(argv: &[&str]) -> CliResult<()> {
             };
         }
         Err(e) => {
-            #[cfg(all(target_os = "linux", feature = "magic"))]
-            {
-                sniffing_error = Some(format!("{e}. Detected mime type: {file_type}"));
-            }
-            #[cfg(not(feature = "magic"))]
-            {
-                sniffing_error = Some(format!("{e}"));
-            }
+            sniff_error = Some(format!("{e}"));
         }
     }
 
     cleanup_tempfile(sfile_info.tempfile_flag, tempfile_to_delete)?;
 
+    // safety: we just created all these json values above, so they are safe to unwrap
     if args.flag_json || args.flag_pretty_json {
-        if sniffing_error.is_none() {
+        if sniff_error.is_none() {
             if args.flag_pretty_json {
                 println!(
                     "{}",
@@ -989,18 +1062,48 @@ pub async fn run(argv: &[&str]) -> CliResult<()> {
             };
             Ok(())
         } else {
-            let json_error = json!({
-                "errors": [{
+            #[allow(unused_assignments)]
+            let mut sniff_error_json: serde_json::Value = Value::default();
+            #[cfg(all(target_os = "linux", feature = "magic"))]
+            {
+                sniff_error_json = json!({
                     "title": "sniff error",
-                    "detail": sniffing_error.unwrap()
-                }]
-            });
-            fail_clierror!("{json_error}")
+                    "detail": format!("{}", sniff_error.unwrap()),
+                    "meta": {
+                        "detected_mime_type": file_type,
+                        "size": sfile_info.file_size,
+                        "last_modified": sfile_info.last_modified,
+                    }
+                });
+            }
+            #[cfg(not(feature = "magic"))]
+            {
+                sniff_error_json = json!({
+                    "title": "sniff error",
+                    "detail": format!("{}", sniff_error.unwrap()),
+                    "meta": {
+                        "detected_mime_type": "Unknown",
+                        "size": sfile_info.file_size,
+                        "last_modified": sfile_info.last_modified,
+                    }
+                });
+            }
+            let error_msg = if args.flag_pretty_json {
+                serde_json::to_string_pretty(&sniff_error_json).unwrap()
+            } else {
+                serde_json::to_string(&sniff_error_json).unwrap()
+            };
+            fail_clierror!("{error_msg}")
         }
-    } else if sniffing_error.is_none() {
+    } else if sniff_error.is_none() {
         println!("{processed_results}");
-        return Ok(());
+        Ok(())
     } else {
-        return fail_clierror!("{}", sniffing_error.unwrap());
+        fail_clierror!(
+            "{error}; size: {size}, last_modified: {last_modified}",
+            error = sniff_error.unwrap(),
+            size = sfile_info.file_size,
+            last_modified = sfile_info.last_modified
+        )
     }
 }
