@@ -49,10 +49,6 @@ pub fn num_cpus() -> usize {
 const CARGO_BIN_NAME: &str = env!("CARGO_BIN_NAME");
 const CARGO_PKG_VERSION: &str = env!("CARGO_PKG_VERSION");
 
-fn default_user_agent() -> String {
-    format!("{CARGO_BIN_NAME}/{CARGO_PKG_VERSION} ({TARGET}; https://github.com/jqnatividad/qsv)")
-}
-
 const TARGET: &str = match option_env!("TARGET") {
     Some(target) => target,
     None => "Unknown_target",
@@ -62,6 +58,10 @@ const QSV_KIND: &str = match option_env!("QSV_KIND") {
     None => "installed",
 };
 
+fn default_user_agent() -> String {
+    format!("{CARGO_BIN_NAME}/{CARGO_PKG_VERSION} ({TARGET}; {QSV_KIND}; https://github.com/jqnatividad/qsv)")
+}
+
 pub fn max_jobs() -> usize {
     let num_cpus = num_cpus();
     let max_jobs = match env::var("QSV_MAX_JOBS") {
@@ -105,8 +105,10 @@ pub fn timeout_secs(timeout: u16) -> Result<u64, String> {
     Ok(timeout as u64)
 }
 
-/// sets user agent
+/// sets custom user agent
 /// if user agent is not set, then use the default user agent
+/// it supports four special LITERALs: $QSV_BIN_NAME, $QSV_VERSION, $QSV_TARGET, and $QSV_KIND
+/// which will be replaced with the actual values during runtime
 pub fn set_user_agent(user_agent: Option<String>) -> CliResult<String> {
     use reqwest::header::HeaderValue;
 
@@ -115,6 +117,13 @@ pub fn set_user_agent(user_agent: Option<String>) -> CliResult<String> {
         None => env::var("QSV_USER_AGENT").unwrap_or_else(|_| default_user_agent()),
     };
 
+    // look for special literals - $QSV_VERSION and $QSV_TARGET and replace them
+    let ua = ua
+        .replace("$QSV_BIN_NAME", CARGO_BIN_NAME)
+        .replace("$QSV_VERSION", CARGO_PKG_VERSION)
+        .replace("$QSV_TARGET", TARGET)
+        .replace("$QSV_KIND", QSV_KIND);
+
     match HeaderValue::from_str(ua.as_str()) {
         Ok(_) => (),
         Err(e) => return fail_clierror!("Invalid user-agent value: {e}"),
@@ -1420,19 +1429,9 @@ pub fn sniff_filetype_from_buffer(in_buffer: &bytes::Bytes) -> Result<String, Cl
         buffer_wrk.extend_from_slice(&in_buffer[0..buffer_len]);
         let mime = cookie.buffer(&buffer_wrk)?;
 
-        if log_enabled!(log::Level::Debug) {
-            log::debug!(
-                "scanned first {buffer_len} bytes - detected mime: {}",
-                mime.clone()
-            );
-        }
-
         Ok(mime)
     } else {
-        // if we have no bytes, we can't determine the mime type
-        // so we return application/octet-stream per
-        // https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types#important_mime_types_for_web_developers
-        // and https://stackoverflow.com/questions/1176022/unknown-file-type-mime
-        Ok("application/octet-stream".to_string())
+        // empty file, so we return inode/x-empty
+        Ok("inode/x-empty".to_string())
     }
 }
